{*************************************************************}
{            Procesor Components for Delphi 32                }
{ Version:   1.00                                             }
{ Author:    Aleksey Kuznetsov, Kiev, Ukraine                 }
{              (Xacker), ,          }
{ E-Mail:    xacker@phreaker.net                              }
{ Home Page: xacker.virtualave.net                            }
{ Created:   January, 19, 1999                                }
{ Modified:  January, 19, 1999                                }
{ Legal:     Copyright (c) 1999 by Aleksey Xacker             }
{*************************************************************}
{   TimeProcessor (English):                                  }
{ This package contains two components: TTimeProcessor and    }
{ TChildProcess. They are intended for differentiation of     }
{ time necessary for processing any tasks. TTimeProcessor     }
{ (master) in a cycle, on turn gives time to each of          }
{ TChildProcess (slave). For example, me these components     }
{ ware necessary for a sending inquiries and processing the   }
{ answers from the external device (through Comm port) from   }
{ various windows of the program.                             }
{*************************************************************}
{   TimeProcessor (Russian):                                  }
{     : TTimeProcessor         }
{ TChildProcess.       }
{    - . TTimeProcessor }
{  ,                }
{ TChildProcess. ,        }
{               }
{  ( Comm )    .   }
{*************************************************************}
{ TTimeProcessor (extended TTimer)                            }
{ Properties: ************************************************}
{            Busy: Flag of readiness of child processes.      }
{   AutoClearBusy: If True then don't use Busy flag property. }
{         Enabled: As it usual...                             }
{        Interval: Interval determines the amount of time, in }
{                  milliseconds, that passes before the timer }
{                  component initiates another OnTimer event  }
{                  and processing childs.                     }
{           Count: Count is the number of childs are in use.  }
{ Events:     ************************************************}
{         OnTimer: OnTimer occurs when a specified amount of  }
{                  time, determined by the Interval property, }
{                  has passed. Like TTimer.OnTimer.           }
{      OnAddChild,                                            }
{   OnRemoveChild: Notifies about Add or Remove TProcessChild.}
{ Methods:    ************************************************}
{             Add: Adds new TChildProcess.                    }
{          Remove: Removes specified TChildProcess.           }
{*************************************************************}
{ TChildProcess                                               }
{ Properties: ************************************************}
{         Enabled: As it usual...                             }
{   TimeProcessor: Pointer on 'parent' TTimeProcessor.        }
{ Events:     ************************************************}
{       OnProcess: Time is given.                             }
{ Methods:    ************************************************}
{           Ready: Turn On readiness of parent TTimeProcessor }
{                  for processing further processes. Is used  }
{                  only if at the 'parent'                    }
{                  AutoClearBusy = False. After calling of a  }
{                  method Ready, at the 'parent' a Busy flag  }
{                  is cleared (same as Busy = False).         }
{*************************************************************}
{ Also see demonstration program.                             }
{*************************************************************}
{ Thanks for using TimeProcessor components.                  }
{ If at occurrence of any questions concerning these          }
{ components, mail me: xacker@phreaker.net.                   }
{*************************************************************}

unit Procesor;

interface

uses
  Windows, Messages, Classes, Controls, Forms;

type
  TTimeProcessor = class(TComponent)
  private
    FBusy: Boolean;
    FAutoClearBusy: Boolean;
    FList: TList;
    CurrentItem: Integer;

    FInterval: Cardinal;
    FWindowHandle: HWND;
    FOnTimer: TNotifyEvent;
    FOnAddChild, FOnRemoveChild: TNotifyEvent;
    FEnabled: Boolean;
    procedure UpdateTimer;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure SetOnTimer(Value: TNotifyEvent);
    procedure WndProc(var Msg: TMessage);
  protected
    procedure Timer; dynamic;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    procedure Add(Child: TComponent);
    procedure Remove(Child: TComponent);
    function Count: Integer;
  published
    property Busy: Boolean read FBusy write FBusy;
    property AutoClearBusy: Boolean read FAutoClearBusy write FAutoClearBusy;
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property OnTimer: TNotifyEvent read FOnTimer write SetOnTimer;
    property OnAddChild: TNotifyEvent read FOnAddChild write FOnAddChild;
    property OnRemoveChild: TNotifyEvent read FOnRemoveChild write FOnRemoveChild;
  end;

  TChildProcess = class(TComponent)
  private
    FEnabled: Boolean;
    FTimeProcessor: TTimeProcessor;
    FOnProcess: TNotifyEvent;

    procedure SetTimeProcessor(Value: TTimeProcessor);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Ready;
  published
    property Enabled: Boolean read FEnabled write FEnabled;
    property TimeProcessor: TTimeProcessor read FTimeProcessor write SetTimeProcessor;
    property OnProcess: TNotifyEvent read FOnProcess write FOnProcess;
  end;

procedure Register;

implementation

{ TTimeProcessor }

constructor TTimeProcessor.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FAutoClearBusy := True;

  FEnabled := True;
  FInterval := 1000;
  FWindowHandle := AllocateHWnd(WndProc);
  UpdateTimer;

  FList := TList.Create;
end;

destructor TTimeProcessor.Destroy;
begin
  FEnabled := False;
  UpdateTimer;
  DeallocateHWnd(FWindowHandle);

  FList.Free;
  inherited Destroy;
end;

procedure TTimeProcessor.WndProc(var Msg: TMessage);
begin
  with Msg do
    if Msg = WM_TIMER then
      try
        Timer;
      except
        Application.HandleException(Self);
      end
    else
      Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;

procedure TTimeProcessor.UpdateTimer;
begin
  KillTimer(FWindowHandle, 1);
  if (FInterval <> 0) and FEnabled then
    if SetTimer(FWindowHandle, 1, FInterval, nil) = 0 then
      raise EOutOfResources.Create('No timers');
end;

procedure TTimeProcessor.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TTimeProcessor.SetInterval(Value: Cardinal);
begin
  if Value <> FInterval then
  begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

procedure TTimeProcessor.SetOnTimer(Value: TNotifyEvent);
begin
  FOnTimer := Value;
  UpdateTimer;
end;

procedure TTimeProcessor.Timer;
var
  CurrentChild: TChildProcess;
  Processed: Boolean;
  Tries: Integer;
begin
  Tries := 0;
  Processed := False;
  if FList.Count <> 0 then
   repeat
    if CurrentItem >= FList.Count then CurrentItem := 0;
    CurrentChild := TChildProcess(FList.Items[CurrentItem]);

    if CurrentChild <> nil then
     if CurrentChild.Enabled and Assigned(CurrentChild.OnProcess) and
        (CurrentChild.TimeProcessor = Self) then
      begin
       CurrentChild.OnProcess(Self);
       Processed := True;
      end;

    inc(CurrentItem);
    inc(Tries);
   until Processed or (Tries = FList.Count);

  if Assigned(FOnTimer) then FOnTimer(Self);
end;

procedure TTimeProcessor.Add(Child: TComponent);
begin
  if not (csDesigning in ComponentState) then
   begin
    FList.Add(Child);
    if Assigned(FOnAddChild) then FOnAddChild(Self);
   end;
end;

procedure TTimeProcessor.Remove(Child: TComponent);
var
  i: Integer;
  CurrentChild: TComponent;
begin
  if FList.Count <> 0 then
   for i := 0 to FList.Count - 1 do
    begin
     CurrentChild := FList.Items[i];
     if CurrentChild = Child then
      begin
       FList.Remove(CurrentChild);
       FList.Pack;
       if Assigned(FOnRemoveChild) then FOnRemoveChild(Self);
       Exit;
      end;
    end;
end;

function TTimeProcessor.Count: Integer;
begin
  Result := FList.Count;
end;

{ TChildProcess }

constructor TChildProcess.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := True;
end;

destructor TChildProcess.Destroy;
begin
  if FTimeProcessor <> nil then FTimeProcessor.Remove(Self);
  inherited Destroy;
end;

procedure TChildProcess.Ready;
begin
  if FTimeProcessor <> nil then
   FTimeProcessor.Busy := False;
end;

procedure TChildProcess.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = TimeProcessor) and
     (Operation = opRemove) then
   TimeProcessor := nil;
end;

procedure TChildProcess.SetTimeProcessor(Value: TTimeProcessor);
begin
  if FTimeProcessor <> Value then
   begin
    if FTimeProcessor <> nil then FTimeProcessor.Remove(Self);
    FTimeProcessor := Value;
    if Value <> nil then
     begin
      Value.FreeNotification(Self);
      Value.Add(Self);
     end;
   end;
end;

{ ----------- }

procedure Register;
begin
  RegisterComponents('Xacker', [TTimeProcessor, TChildProcess]);
end;

end.
